# -*- coding: utf-8 -*-
"""PAiP Web Build System - Program Class
This module contains the most core functionality,
needed by other functions and classes.

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Imports
import argparse as ap
from . import __version__ as pwbs_version
from .config.config_manager import PWBSConfigFileDontExistError
from .config.config_manager import PWBSInvalidConfigFile
from .config.config_manager import ConfigManager
from .config.pwbs_config import PWBS_ConfigManager as PWBS_CM
from .core import NotImplementedFeatureError
from .tests import test_runner
# from .lib.pwm.pwm_system import SystemVersion #TODO: DEV:

# Underscore Variables

"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'

# Main Program Class


class PWBS(object):
    """PAiP Web Build System - Program Class"""
    def __init__(self):
        """Constructor of the Main Program Class"""
        """Argument Parser"""
        self.argparser = ap.ArgumentParser(
            formatter_class=ap.RawTextHelpFormatter,
            prog="pwbs",
            usage="%(prog)s [Options] Tasks",
            description="PAiP Web Build System v.{0}".format(pwbs_version),
            epilog="Build System based on JSON configuration files.")
        self.parser_initializer()
        """PWBS Config Manager"""
        self.pwbscm = PWBS_CM()
        """Try for errors"""
        if isinstance(
                self.pwbscm.configmanager.error,
                PWBSConfigFileDontExistError):
            print(
                "Warning: Configuration File Doesn't Exist!")
            self.pwbscm.log.log_debug(
                "CALLER: pwbs.pwbs_class.PWBS.__init__()")
            self.pwbscm.log.log_debug(repr(self.pwbscm.configmanager.error))
        elif isinstance(
                self.pwbscm.configmanager.error,
                PWBSInvalidConfigFile):
            print(
                "Warning: Configuration File is Invalid")
            self.pwbscm.log.log_debug(
                "CALLER: pwbs.pwbs_class.PWBS.__init__()")
            self.pwbscm.log.log_debug(repr(self.pwbscm.configmanager.error))
        self.localconfig_parser_initializer()
        """Arguments returned from argparse.parse_args()"""
        self.args = None

    def parser_initializer(self):
        """Parser Initializer"""
        self.argparser_specialtasks = self.argparser.add_argument_group(
            "Special Tasks",
            "Special Bultin PWBS Functions and Flags")
        self.argparser_specialtasks.add_argument(
            "-v",
            "--verbose",
            default="1",
            choices=["0", "1", "2", "3", "255"],
            required=False,
            help="""Changing Verbosity:
            \n0 - No Information about running
            \n1 - Normal Information about running [Default]
            \n2 - More Information about running
            \n3 - Much Information about running and work behind the scenes
            \n255 - Debug Verbosity Mode""")
        self.argparser_specialtasks.add_argument(
            "--debug",
            default=False,
            action="store_true",
            required=False,
            help="""Changing Debug Mode:
            \nDebug Mode Turned On [Good with -v 255]
            \nDebug Mode Turned Off [Default]""")
        self.argparser_specialtasks.add_argument(
            "--version",
            action="store_true",
            required=False,
            help="""Showing Version of PWBS""")
        self.argparser_specialtasks.add_argument(
            "--new-config",
            action="store_true",
            required=False,
            help="""Creating Blank Configuration File from basic template""")
        self.argparser_specialtasks.add_argument(
            "-l",
            "--log",
            action="store_true",
            required=False,
            help="""Enabling Logging To File""")
        self.argparser_specialtasks.add_argument(
            "-lf",
            "--logfile",
            "--log-file",
            required=False,
            type=ap.FileType("w"),
            help="""Specifying Log File
            \nDefault Log File: ./pwbs.log""")
        self.argparser_specialtasks.add_argument(
            "-c",
            "--configfile",
            "--config-file",
            required=False,
            type=ap.FileType("rw"),
            help="""Specifying Configuration File
            \nDefault Configuration File: ./pwbs.json""")
        self.argparser_specialtasks.add_argument(
            "--test-mode",
            action="store_true",
            required=False,
            help="""Enabling Test Mode
            \nThis option is to use only in tests (automated and manual)""")
        self.argparser_specialtasks.add_argument(
            "--run-tests",
            action="store_true",
            required=False,
            help="""Starting PWBS Self-Testing Module""")
        self.argparser.add_argument(
            "Task",
            nargs="*",
            help="""Task""")
        self.argparser_localconfigtasks = self.argparser.add_argument_group(
            "Local Tasks",
            "Local Configuration Tasks")

    def localconfig_parser_initializer(self):
        """Local Configuration Parser Initializer"""
        try:
            self.pwbscm.commands_to_commandlist()
            for cmd in self.pwbscm.commands.items():
                arg = cmd.argument_parser()
                self.argparser_localconfigtasks.add_argument(
                    cmd.name,
                    nargs="?",
                    help=arg)
        except PWBSConfigFileDontExistError as e:
            self.pwbscm.log.log_debug(
                "CALLER: pwbs.pwbs_class.PWBS.localconfig_parser_initializer()")
            self.pwbscm.log.log_debug(repr(e))
        except PWBSInvalidConfigFile as e:
            if isinstance(
                    self.pwbscm.configmanager.error,
                    PWBSConfigFileDontExistError):
                self.pwbscm.log.log_debug(
                    "CALLER: pwbs.pwbs_class.PWBS.localconfig_parser_initializer()")
                self.pwbscm.log.log_debug(repr(e))
            else:
                print("Warning: Configuration File is Invalid")
                self.pwbscm.log.log_debug(
                    "CALLER: pwbs.pwbs_class.PWBS.localconfig_parser_initializer()")
                self.pwbscm.log.log_debug(repr(e))

    def special_tasks_interpreter(self):
        """Special Tasks Interpreter"""
        self.pwbscm.log.verbose(int(self.args.verbose))
        self.pwbscm.log.debug(self.args.debug)
        if self.args.version is True:
            print("PAiP Web Build System v.{0}".format(pwbs_version))
        if self.args.log is True:
            self.pwbscm.log.log_logger.activelogging = True
            self.pwbscm.log.log_file_write()
        if self.args.logfile is not None:
            self.pwbscm.log.log_logger.logfile = self.args.logfile
        if self.args.configfile is not None:
            self.pwbscm.config_manager = ConfigManager(self.args.configfile)
            self.localconfig_parser_initializer()
        if self.args.test_mode is True:
            self.pwbscm.log.debug(True)
            self.pwbscm.log.verbose(255)
        if self.args.run_tests is True:
            # TODO: Running Tests DEV: TO_TEST: Not tests yet
            print("Test Runner Mode Activated!")
            test_runner()
            # NOT_IMPLEMENTED:from .core import NotImplementedFeatureError
            # NOT_IMPLEMENTED:raise NotImplementedFeatureError("Not Implemented Feature Called")

    def task_runner(self):
        """Task Runner"""
        for arg in self.args.Task:
            self.pwbscm.commands[arg].run()

    def main(self):
        """Main Function of the Program"""
        try:
            self.args = self.argparser.parse_args()
            self.pwbscm.log.log_debug("Argument Parser: {0}".format(repr(self.args)))
            self.special_tasks_interpreter()
            self.task_runner()
        except NotImplementedFeatureError:
            print("Not Implemented Feature Called!")



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Test

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT

"""
# Imports
import pytest
# Documentation
"""
Test 0
================
Test for test
"""
# Test Function


def test_0_0():
    assert True


@pytest.mark.xfail(raises=AssertionError)
def test_0_1():
    assert False



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Logging Module
This module contains Classes for Loggers and everything needed for Logging.

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT

"""
# Underscore Variables
"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Config Manager

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Imports
from __future__ import print_function
from os import getcwd
from os.path import isfile
from ..lib.pwm.pwm_json import JSON_IO
from ..lib.pwm.pwm_system import SystemVersion

# Class Definition


class PWBSConfigFileDontExistError(AssertionError):
    """Error for handling configuration file that don't exist"""
    pass


class PWBSInvalidConfigFile(KeyError):
    """Error for handling invalid configuration file"""
    pass


class ConfigManager(object):
    """Config Manager Class
    This class is for Managing Config File.
    """
    def __init__(self, filename="pwbs.json"):
        """Constructor of the class"""
        """Configuration File Name"""
        self.config_filename = filename
        # Getting path of configuration file
        cwd = getcwd()
        if SystemVersion().system_system == "Windows":
            """Configuration File Path"""
            self.config_filename_path = cwd+"\\"+self.config_filename
        else:
            """Configuration File Path"""
            self.config_filename_path = cwd+"/"+self.config_filename
        """Test for existence"""
        self.error = None
        try:
            try:
                assert isfile(self.config_filename_path)
            except AssertionError as e:
                raise PWBSConfigFileDontExistError(
                    "PWBS Configuration File Doesn't Exist!") from e
        except PWBSConfigFileDontExistError as e:
            self.error = e
        """Configuration File Manager"""
        self.filemanager = JSON_IO(self.config_filename_path)
        # Preloading File
        self.load()

    def load(self):
        """Configuration Loader
        Reading JSON from Config File and Converting to Dict.
        """
        self.config_dict = dict(self.filemanager.read())
        try:
            try:
                self.config_dict["commands"]
            except KeyError as e:
                raise PWBSInvalidConfigFile(
                    "PWBS Configuration File is invalid!") from e
        except PWBSInvalidConfigFile as e:
            if self.error is None:
                self.error = e
        return self.config_dict

    def write(self, newdata):
        """Configuration Writer
        Writing JSON to Config File.
        Args:
            newdata: New Data for the Config File.
        """
        return self.filemanager.write(newdata)



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Tests Module
This module contains tests in the submodules.
In exacly this file is some functions for runnning tests from code.
READ - README.md in tests for rules of tests

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT

"""
# Global Imports
from datetime import datetime
# Underscore Variables
"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'
# Special Exceptions


class TestSucceed(Exception):
    """Exception For Passed Test"""
    pass


class TestFailed(Exception):
    """Exception For Failed Test"""
    pass

# Special Functions


def tests_prefix_text(text=""):
    """Function to prefix text [Test Runner Version]
    Default Prefixer
    Args:
        text (:obj:`str`): Text to prefix
    """
    return print("PWBS_TEST[{0}]: {1}".format(
        datetime.now().strftime("%H:%M:%S"),
        text))


def run_test(
        test_function,
        test_name: str,
        test_comment: str,
        test_module: str,
        test_except_to_fail=False):
    tests_prefix_text(
        "Testing {0} in Test {1}".format(
            test_module,
            test_name))
    tests_prefix_text(
        "Comment for Test {0}: {1}".format(
            test_name,
            test_comment))
    try:
        try:
            test_function()
            raise TestSucceed(test_name)
        except AssertionError as e:
            if test_except_to_fail:
                raise TestSucceed(test_name) from e
            raise TestFailed(test_name) from e
    except TestSucceed:
        tests_prefix_text(
            "[RESPONSE]: Test {0} - Success!".format(
                test_name))
    except TestFailed as e:
        tests_prefix_text(
            "[RESPONSE]: Test {0} - Fail!".format(
                test_name))
        # Failed Tests Debug
        raise e


def test_runner():
    # ################## Test 0
    from .test_0 import test_0_0, test_0_1
    # # Test 0.0
    run_test(test_0_0,
             "0.0",
             "Test Asserting True",
             "Python",
             False)
    # # Test 0.1
    run_test(test_0_1,
             "0.1",
             "Test Asserting False",
             "Python",
             True)
    # ################## Test 1
    from .test_1 import test_1_0, test_1_1, test_1_2, test_1_3
    from .test_1 import test_1_4, test_1_5, test_1_6
    from .test_1 import test_1_7, test_1_8, test_1_9
    # # Test 1.0
    run_test(test_1_0,
             "1.0",
             "Testing Values",
             "pwbs.command.command.CommandType",
             False)
    # # Test 1.1
    run_test(test_1_1,
             "1.1",
             "Testing Values",
             "pwbs.command.command.Platform",
             False)
    # # Test 1.2
    run_test(test_1_2,
             "1.2",
             "Testing Values",
             "pwbs.command.command.CommandMode",
             False)
    # # Test 1.3
    run_test(test_1_3,
             "1.3",
             "Test Command",
             "pwbs.command.command.Command",
             False)
    # # Test 1.4
    run_test(test_1_4,
             "1.4",
             "Test Command",
             "pwbs.command.command.Command",
             False)
    # # Test 1.5
    run_test(test_1_5,
             "1.5",
             "Test Command",
             "pwbs.command.command.Command",
             False)
    # # Test 1.6
    run_test(test_1_6,
             "1.6",
             "Test Command",
             "pwbs.command.command.Command",
             False)
    # # Test 1.7
    run_test(test_1_7,
             "1.7",
             "Test Command",
             "pwbs.command.command.Command",
             False)
    # # Test 1.8
    run_test(test_1_8,
             "1.8",
             "Test Command",
             "pwbs.command.command.Command",
             False)
    # # Test 1.9
    run_test(test_1_9,
             "1.9",
             "Test CommandList",
             "pwbs.command.command.CommandList",
             False)
    # ################## Test 2
    from .test_2 import test_2_0, test_2_1, test_2_2
    # # Test 2.0
    run_test(test_2_0,
             "2.0",
             "Testing Exceptions",
             "pwbs.config.config_manager.* : Exception",
             False)
    # # Test 2.1
    run_test(test_2_1,
             "2.1",
             "Testing Config Manager",
             "pwbs.config.config_manager.ConfigManager",
             False)
    # # Test 2.2
    run_test(test_2_2,
             "2.2",
             "Testing PWBS Config Manager",
             "pwbs.config.pwbs_config.PWBS_ConfigManager",
             False)
    # ################## Test 3
    from .test_3 import test_3_0, test_3_1
    # # Test 3.0
    run_test(test_3_0,
             "3.0",
             "Testing pwbs.core.NotImplementedFeatureError",
             "pwbs.core.NotImplementedFeatureError",
             False)
    # # Test 3.1
    run_test(test_3_1,
             "3.1",
             "Testing pwbs.core.prefix_text",
             "pwbs.core.prefix_text",
             False)
    # ################## Test 5
    from .test_5 import test_5_0, test_5_1, test_5_2, test_5_3
    # # Test 5.0
    run_test(test_5_0,
             "5.0",
             "Testing pwbs.log.logger.LoggerAssertionError",
             "pwbs.log.logger.LoggerAssertionError",
             False)
    # # Test 5.1
    run_test(test_5_1,
             "5.1",
             "Testing pwbs.log.logger.BaseLogger",
             "pwbs.log.logger.BaseLogger",
             False)
    # # Test 5.2
    run_test(test_5_2,
             "5.2",
             "Testing pwbs.log.logger.LogLogger",
             "pwbs.log.logger.LogLogger",
             False)
    # # Test 5.3
    run_test(test_5_3,
             "5.3",
             "Testing pwbs.log.logger.Logger",
             "pwbs.log.logger.Logger",
             False)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""PAiP Web Module - Debug

This module creates Debug decorator as well as ProgramConfiguration class which can be used to Configure Program and Debug Mode

NAME - PAiP Web Module - Debug
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT
VERSION - v.0.0.0.2

Examples:
    Example 1:
        # Using a ProgramConfiguration Class to handle decorator
        from lib.pwm.pwm_debug import ProgramConfiguration
        ProgramConf = ProgramConfiguration()
        ProgramConf.SetDebugMode(True)
        @ProgramConf.debug
        def main(name):
            return print("Hello", name)
        main("Patryk")
    Example 2:
        # Using decorator directly
        from lib.pwm.pwm_debug import debug
        @debug(True)
        def main(name):
            return print("Hello", name)
        main("Patryk")

Todo:
    * Probably Change ProgramConfiguration Class and make in pwm_programconfig.py

"""
### Imports
# Timing
from time import time
# Getting Source and file of function
from inspect import getsource, getfile
### Functions

def print_debug(name, *args, **kwargs):
    """Printing Debug

    Function to display formatted output in decorator options
    Args:
        name: Name of Process [like "Function Source:"]
        *args: Variable length argument list.
        **kwargs: Arbitrary keyword arguments.
    Returns:
        called print function
    """
    return print("DEBUG|", name, *args, **kwargs)

def debug(
        verbose_mode=False,
        function_name=False,
        argument_listing=False,
        source_listing=False,
        file_listing=False,
        return_listing=False,
        error_listing=False,
        timing_listing=False
    ):
    """Debug Decorator

    It's decorator to debug wrapped function.
    If you want to don't want debug decorator without deleting code use Program Configuration way or use @debug() as decorator [Without paramaters].
    If you want full debug mode use @debug(True).
    If you want specific debug mode than use keyword arguments.

    Args:
        verbose_mode (bool, optional): Verbose Mode. Turns on full debug mode. Defaults to False.
        function_name (bool, optional): Printing to Command Line wrapped function name. Defaults to False.
        argument_listing (bool, optional): Printing to Command Line wrapped function parameters. Defaults to False.
        source_listing (bool, optional): Printing to Command Line wrapped function source code. Defaults to False.
        file_listing (bool, optional): Printing to Command Line in what file wrapped function is. Defaults to False.
        return_listing (bool, optional): Printing to Command Line wrapped function return. Defaults to False.
        error_listing (bool, optional): Printing to Command Line wrapped function error. If False then try code and catch exception raising another. Defaults to False.
        timing_listing (bool, optional): Printing to Command Line wrapped function timing. Defaults to False.

    """
    def f1(f):
        def f2(*args, **kwargs):
            if function_name or verbose_mode:
                print_debug("Function Name:", f.__name__)
            if argument_listing or verbose_mode:
                print_debug("Function Args:", str(args), str(kwargs))
            if source_listing or verbose_mode:
                print_debug("Function Source:\n", getsource(f))
            if file_listing or verbose_mode:
                print_debug("Function is in:", getfile(f))
            try:
                before = time()
                ret = f(*args, **kwargs)
                after = time()
                elapsed = after - before
                if timing_listing or verbose_mode:
                    print_debug("Elapsed time:", elapsed)
            except Exception as ex:
                if error_listing or verbose_mode:
                    print_debug("Catched:", str(ex))
                else:
                    raise ex
            if return_listing or verbose_mode:
                print_debug("Function Returned:", ret)
            return ret
        return f2
    return f1



# -*- coding: utf-8 -*-
"""PAiP Web Build System

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Underscore Variables

"""Title of Program"""
__title__ = 'pwbs'
"""Version of Program"""
__version__ = '0.4.0-alpha0'
"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'

# Imports

from .pwbs_class import PWBS

# Running as pwbs command


def main():
    pwbs_class_var = PWBS()
    pwbs_class_var.main()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Test

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT

"""
# Documentation
"""
Test 1
================
Tests for v0.4.0dev0 | pwbs.command
================
Test 1.0 - CommandType(Enum)
Test 1.1 - Platform(Enum)
Test 1.2 - CommandMode(Enum)
Test 1.3 - 1.8 - Command(object)
Test 1.9 - CommandList(object)
"""
# Test Function


def test_1_0():
    """Testing CommandType"""
    # Import
    from ..command.command import CommandType
    # Values
    assert CommandType.NullTask == CommandType(0x000)
    assert CommandType.InitialNullTask == CommandType(0x001)
    assert CommandType.SpecialTask == CommandType(0x101)
    assert CommandType.SingleTask == CommandType(0x201)
    assert CommandType.MultiTask == CommandType(0x202)
    assert CommandType.MultiCommandTask == CommandType(0x203)
    assert CommandType.WatcherTask == CommandType(0x204)
    assert CommandType.SchedulerTask == CommandType(0x205)
    assert CommandType.TestTask == CommandType(0xFF0)
    assert CommandType.ErrorTask == CommandType(0xFF1)
    assert CommandType.NullTaskF == CommandType(0xFFF)


def test_1_1():
    """Testing Platform"""
    # Import
    from ..command.command import Platform
    # Values
    assert Platform.NullOS == Platform(0b0000)
    assert Platform.Windows == Platform(0b0001)
    assert Platform.Linux == Platform(0b0010)
    assert Platform.MacOS == Platform(0b0100)
    assert Platform.Other == Platform(0b1000)
    assert Platform.Any == Platform(0b1111)


def test_1_2():
    """Testing CommandMode"""
    # Import
    from ..command.command import CommandMode
    # Values
    assert CommandMode.SingleTask_Standard == CommandMode(0x22010001)
    assert CommandMode.MultiTask_Standard == CommandMode(0x22020001)
    assert CommandMode.MultiCommandTask_Standard == CommandMode(0x22030001)
    assert CommandMode.WatcherTask_StartAndRun == CommandMode(0x22040001)
    assert CommandMode.WatcherTask_StartAndWait == CommandMode(0x22040002)
    assert CommandMode.SchedulerTask_StartAndRun == CommandMode(0x22050001)
    assert CommandMode.SchedulerTask_StartAndWait == CommandMode(0x22050002)
    assert CommandMode.NullMode == CommandMode(0x00000000)
    assert CommandMode.ModeNotSpecified == CommandMode(0x00000001)
    assert CommandMode.TestTask_TestMode == CommandMode(0xFFF00000)
    assert CommandMode.ErrorTask_ErrorMode == CommandMode(0xFFF1FFFF)
    assert CommandMode.NullTask_NullMode == CommandMode(0xF000FFF0)
    assert CommandMode.NullTaskF_NullMode == CommandMode(0xFFFFFFF0)
    assert CommandMode.ErrorMode == CommandMode(0xFFFFFFFF)


def test_1_3():
    """Testing Command"""
    # Import
    from ..command.command import Command
    # Tests
    # Command.__init__()
    test_variable = Command("test_1_3")
    assert test_variable is not None
    test_variable._log.log("Test 1.3")
    # Command.__init__() Variables and Defaults
    from ..log.logger import Logger
    assert isinstance(test_variable._log, Logger)
    assert test_variable.name == "test_1_3"
    from ..command.command import CommandType, CommandMode, Platform
    assert test_variable.type == CommandType.InitialNullTask
    assert test_variable.commands is None
    assert test_variable.comment == ""
    assert test_variable.mode == CommandMode.NullMode
    assert test_variable.arguments is None
    assert test_variable.special is None
    assert test_variable.platform == Platform.Any
    # Command.__eq__()
    test_variable__2 = Command("test_1_3")
    assert test_variable == test_variable__2
    assert test_variable == "test_1_3"
    assert test_variable__2 == "test_1_3"


def test_1_4():
    """Testing Command"""
    # Import
    from ..command.command import Command, CommandType, CommandMode
    # Command.run()
    # TO_TEST: More Tests on that
    test_variable__3 = Command(
        "test_1_3",
        CommandType.TestTask,
        None,
        "",
        CommandMode.TestTask_TestMode)
    assert test_variable__3.run() == "TESTED"
    # Command.argument_parser()
    assert test_variable__3.comment == test_variable__3.argument_parser()


def test_1_5():
    """Testing Command"""
    # Import
    from ..command.command import Command, CommandType, CommandMode
    # Command.execute_as_singletask_or_multicommand()
    test_variable__6 = Command("test_1_3", CommandType.SingleTask, ["echo If you see this that means test 1.3 Works!"], "Test 1.3", CommandMode.SingleTask_Standard)
    test_variable__6.run()


def test_1_6():
    """Testing Command"""
    # Import
    from ..command.command import Command, CommandType, CommandMode
    # Command.execute_as_watcher()
    test_variable__7 = Command(
        "test_1_3",
        CommandType.TestTask,
        None,
        "",
        CommandMode.TestTask_TestMode)
    from ..core import NotImplementedFeatureError
    try:
        test_variable__7.execute_as_watcher()
    except NotImplementedFeatureError:
        pass


def test_1_7():
    """Testing Command"""
    # Import
    from ..command.command import Command, CommandMode, CommandType
    # Command.execute_as_scheduler()
    test_variable__8 = Command(
        "test_1_3",
        CommandType.TestTask,
        None,
        "",
        CommandMode.TestTask_TestMode)
    from ..core import NotImplementedFeatureError
    try:
        test_variable__8.execute_as_watcher()
    except NotImplementedFeatureError:
        pass


def test_1_8():
    """Testing Command"""
    # Import
    from ..command.command import Command, CommandType
    # Command.__add__()
    test_table = ["t1", "t2"]
    test_variable__4 = Command("test_1_3", CommandType.TestTask, [test_table[0]])
    test_variable__5 = Command("test_1_3", CommandType.TestTask, [test_table[1]])
    test_var = (test_variable__4.commands + test_variable__5.commands)
    test_res = (test_variable__4 + test_variable__5)
    assert test_res.commands == test_var
    assert test_res.commands == test_table


def test_1_9():
    """Testing CommandList"""
    # Import
    from ..command.command import Command, CommandList
    # Test Vars
    test_var_1 = Command("test_1_9_1")
    test_var_2 = Command("test_1_9_2")
    test_var_3 = Command("test_1_9_3")
    test_var_4 = Command("test_1_9_4")
    test_var_5 = Command("test_1_9_5")
    # CommandList.__init__()
    test_table = [test_var_1, test_var_2, test_var_3]
    test_var = CommandList(test_table)
    assert test_var.values == test_table
    # CommandList.__getitem__()
    for i in range(0, len(test_table)):
        assert test_var[test_table[i].name].name == test_table[i].name
    # CommandList.__setitem__()
    test_var[test_var_3.name] = test_var_5
    assert test_var[test_var_5.name].name == test_var_5.name
    # # New Item
    test_var[test_var_4.name] = test_var_4
    print(test_var[test_var_4.name].name)
    # CommandList.__delitem__()
    del test_var[test_var_4.name]
    try:
        test = test_var[test_var_4.name]
        assert False
        assert test is not None
    except KeyError:
        pass
    # CommandList.__contains__()
    assert test_var_1.name in test_var
    # CommandList.items()
    assert test_var.items() is test_var.values



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""PAiP Web Module - Watcher

This module creates functions to watch files as well as Hashing Files.
Watching files working by change in hash of files.

NAME - PAiP Web Module - Watcher
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT
VERSION - v.0.0.0.1
"""
# Imports
import hashlib
import os
import fnmatch
from datetime import datetime
# Functions
def __HashMessage():
    """Function for Print Message every HashFiles Function run
    """
    print("\r[{0}]: Hashing Files...".format(datetime.now().strftime("%H:%M:%S")), end="")
def __HashNull():
    """Null Function for HashFiles Function
    """
    pass

def md5(fname):
    """Function to make md5 hash of file
    Args:
        fname (object): File Name

    Returns:
        :obj:`str`: Returning Hash of File
    """
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()

def findFiles(path_to_watch, arg_files):
    """Function to find files that matching pattern
    Args:
        path_to_watch (object): Path of starting recursive search of files
        arg_files (object): Filename Regular Expression String (like "*.py")

    Returns:
        :obj:`str`: Returning Matches of search
    """
    matches = []
    for root, dirnames, filenames in os.walk(path_to_watch):
        for filename in fnmatch.filter(filenames, arg_files):
            matches.append(os.path.join(root, filename))
    return matches

def HashFiles(path_to_watch, filenames_to_watch, excluded=[], message=__HashNull):
    """Function to hash files
    Args:
        path_to_watch (object): Path of starting recursive search of files
        filenames_to_watch (object): Filename Regular Expression String (like "*.py")
        excluded (object): Excluded items from hashing
        message (function): Function to do before hashing [Can be print]

    Returns:
        :obj:`list`: Returning List of Files Hashes
    """
    message()
    watch_files = findFiles(path_to_watch, filenames_to_watch)
    watch_files = [item for item in watch_files if item not in excluded]
    hash_table = []
    for x in watch_files:
        hash_table.append(md5(x))
    return hash_table

def watchFile(path_to_watch, filenames_to_watch, excluded={}, function=print):
    """Function to watch file for changes
    TODO: CODE CHANGE
    """
    ex2 = []
    for ex, ex1 in excluded.items():
        ex3 = findFiles(ex, ex1)
        ex2 += ex3
    excluded = ex2
    while True:
        lastHash = HashFiles(path_to_watch, filenames_to_watch, excluded)
        while lastHash == HashFiles(path_to_watch, filenames_to_watch, excluded):
            try:
                with open("stop.watching.file.1234567890", "r") as f:
                    #print("PWCIW: Locking File Exist")
                    raise Exception("STOP")
            except FileNotFoundError:
                continue
        try:
            with open("stop.watching.file.1234567890", "r") as f:
                #print("PWCIW: Locking File Exist")
                raise Exception("STOP")
        except FileNotFoundError:
            pass
        #print("\nPWCIW: Hash Changed")
        function()



# -*- coding: utf-8 -*-
"""PAiP Web Build System

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Underscore Variables
__author__ = 'Patryk Adamczyk'
__license__ = 'MIT'
__docformat__ = 'restructuredtext en'



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Test

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT

"""
# Documentation
"""
Test 2
================
Tests for v0.4.0dev0 | pwbs.config
================
Test 2.0 - 2.1 - pwbs.config.config_manager.*
# Test 2.0 - Exceptions Tests
# Test 2.1 - Config Manager Tests
# Test 2.2 - pwbs.config.pwbs_config.PWBS_ConfigManager
"""
# Test Function


def test_2_0():
    """Testing Exceptions"""
    # Import
    from ..config.config_manager import PWBSConfigFileDontExistError
    from ..config.config_manager import PWBSInvalidConfigFile
    # Test
    try:
        raise PWBSConfigFileDontExistError()
        assert False
    except PWBSConfigFileDontExistError:
        assert True
    try:
        raise PWBSInvalidConfigFile()
        assert False
    except PWBSInvalidConfigFile:
        assert True

def test_2_1():
    """Testing Config Manager"""
    # Import
    from ..config.config_manager import ConfigManager
    from ..config.config_manager import PWBSConfigFileDontExistError
    from ..config.config_manager import PWBSInvalidConfigFile
    # Values
    try:
        t1 = ConfigManager("pwbs.json")
        t1.write(t1.load())
    except PWBSConfigFileDontExistError:
        assert True # It's Working
    except PWBSInvalidConfigFile:
        assert True # It's Working


def test_2_2():
    """Testing PWBS Config Manager"""
    # Import
    from ..config.pwbs_config import PWBS_ConfigManager
    from ..config.config_manager import PWBSConfigFileDontExistError
    from ..config.config_manager import PWBSInvalidConfigFile
    # Values
    t1 = PWBS_ConfigManager()
    t1.log.log("Test 2.2")
    try:
        t1.config_file()
        t1.commands_to_commandlist()
    except PWBSConfigFileDontExistError:
        assert True # It's Working
    except PWBSInvalidConfigFile:
        assert True # It's Working



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Command Class

This module contains Command Class and other classes needed by Command Class.
Command Class is for making Command [Tasks] as easy as possible.

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Imports
from __future__ import print_function
from enum import Enum
from datetime import datetime
from ..lib.pwm.pwm_exec import execute_generator
from ..log.logger import Logger

# Class Definition


class CommandType(Enum):
    """Command Type Enum
    This enum is for specifying Command Type in the Command Class.
    """
    # DEV: 0xABB | A - Type | B - Number

    """Null Task"""
    NullTask = 0x000
    """Initial Value - Null Task Type"""
    InitialNullTask = 0x001
    """Special Tasks (like --help)"""
    SpecialTask = 0x101
    # NOT_IMPLEMENTED:
    # """Internal Plugin Task
    # [Plugin task from plugin bultin in PWBS]"""
    # InternalPluginTask = 0x102
    # NOT_IMPLEMENTED:
    # """Plugin Task
    # [Plugin task from plugin written in Python using PWBS Public API]"""
    # PluginTask = 0x103
    # NOT_IMPLEMENTED:
    # """JSON Plugin Task
    # [Plugin task from plugin written in JSON using PWBS JSON Plugin File]"""
    # JSONPluginAPIPlugin = 0x104
    """Single Task
    [One Run = One Shell Command]"""
    SingleTask = 0x201
    """Multi Task
    [One Run = Multiple Tasks]"""
    MultiTask = 0x202
    """Multi Command Task
    [One Run = Multiple Shell Commands]"""
    MultiCommandTask = 0x203
    """Watcher Task"""
    WatcherTask = 0x204
    """Scheduler Task"""
    SchedulerTask = 0x205
    """Test Task
    [For Testing Purposes]"""
    TestTask = 0xFF0
    """Error Task
    [For Error Catching Purposes]"""
    ErrorTask = 0xFF1
    """Null Task"""
    NullTaskF = 0xFFF


class Platform(Enum):
    """Platform Enum
    This enum is to specify platform command is for.
    """
    """Null"""
    NullOS = 0b0000
    """Windows"""
    Windows = 0b0001
    """Linux"""
    Linux = 0b0010
    """Mac OS X"""
    MacOS = 0b0100
    """Any OS not specified above"""
    Other = 0b1000
    """Any OS [default]"""
    Any = 0b1111


class CommandMode(Enum):
    """Command Mode Enum
    This enum is to specify in what mode Command have to run.
    """
    # DEV: 0xABBBCDDD | A - Type | B - Task Type | C - Mode Type | D - Number
    # Task Modes
    """Single Task - Standard Mode"""
    SingleTask_Standard = 0x22010001
    """Multi Task - Standard Mode"""
    MultiTask_Standard = 0x22020001
    """Multi Command Task - Standard Mode"""
    MultiCommandTask_Standard = 0x22030001
    """Watcher Task - Run First Mode"""
    WatcherTask_StartAndRun = 0x22040001
    """Watcher Task - Wait First Mode"""
    WatcherTask_StartAndWait = 0x22040002
    """Scheduler Task - Run First Mode"""
    SchedulerTask_StartAndRun = 0x22050001
    """Scheduler Task - Wait First Mode"""
    SchedulerTask_StartAndWait = 0x22050002
    # Special and Control Modes
    """Null Mode
    [For Testing Purposes]"""
    NullMode = 0x00000000
    """Mode Not Specified
    [For Error Catching Purposes]"""
    ModeNotSpecified = 0x00000001
    """Test Task - Test Mode
    [For Testing Purposes]"""
    TestTask_TestMode = 0xFFF00000
    """Error Task - Error Mode
    [For Error Catching Purposes]"""
    ErrorTask_ErrorMode = 0xFFF1FFFF
    """Null Task - Null Mode
    [For Testing Purposes]"""
    NullTask_NullMode = 0xF000FFF0
    """Null Task - Null Mode
    [For Testing Purposes]"""
    NullTaskF_NullMode = 0xFFFFFFF0
    """Error Mode
    [For Error Catching Purposes]"""
    ErrorMode = 0xFFFFFFFF


class Command(object):
    """Command Class
    This class is for making Commands [Tasks].
    This class is for making Commands File Interpreter too.
    """
    def __init__(
            self,
            name,
            cmd_type: CommandType = CommandType.InitialNullTask,
            commands: list = None,
            comment: str = "",
            mode: CommandMode = CommandMode.NullMode,
            arguments: dict = None,
            special: dict = None,
            platform: Platform = Platform.Any) -> None:
        """Constructor of the Class
        Args:
            name: Name of the Command [Task]
            cmd_type (:obj:`CommandType`): Type of the Command [Task]
                Defaults to Initial Null Task.
            commands (list): List of Commands what Command should do on run
                Defaults to None.
            comment (:obj:`str`): Comment of Command
                Defaults to Empty String
            mode (:obj:`CommandType`): Mode of the Command [Task]
                Defaults to NullMode.
            arguments (dict): Arguments of the Command [Task]
                Defaults to None.
            special (dict): Special Arguments of the Command [Task]
                Defaults to None.
            platform (:obj:`Platform`): Platform of the Command [Task]
                Defaults to Any Platform.
        """
        # Control Vars
        """Logger Variable"""
        self._log = Logger()
        # Defining Variables
        """Name of the Command"""
        self.name = name
        """Type of the Command"""
        self.type = cmd_type
        """List of Commands what Command should do on run"""
        self.commands = commands
        """Comment of Command"""
        self.comment = comment
        """Mode of the Command"""
        self.mode = mode
        """Arguments of the Command"""
        self.arguments = arguments
        """Special Arguments of the Command"""
        self.special = special
        """Platform of the Command"""
        self.platform = platform

    def __eq__(self, other):
        """Equation Operator Overload Function"""
        # Comparing names if it's Command Class
        if isinstance(other, Command):
            return self.name == other.name
        # Comparing name to other value if it's not Command Class
        return self.name == other

    def run(self):
        """Run Function of Command"""
        # Type Checking and running special function for type
        if self.type is CommandType.SingleTask:
            # Single Task
            self.execute_as_singletask_or_multicommand()
        elif self.type is CommandType.MultiCommandTask:
            # Multi Command Task
            self.execute_as_singletask_or_multicommand()
        elif self.type is CommandType.WatcherTask:
            # Watcher Task
            self.execute_as_watcher()
        elif self.type is CommandType.SchedulerTask:
            # Scheduler Task
            self.execute_as_scheduler()
        elif self.type is CommandType.TestTask:
            # Test Task
            return "TESTED"
        elif self.type is CommandType.ErrorTask:
            # Error Task
            raise Exception("Error Task Reached!")
        elif self.type is CommandType.NullTask:
            # Null Task - Debug
            self._log.log_verbose("NullTask Reached!", 2)
        elif self.type is CommandType.NullTaskF:
            # Null Task - Debug
            self._log.log_verbose("NullTask Reached!", 2)
        elif self.type is CommandType.InitialNullTask:
            # Null Task - Debug
            self._log.log_verbose("NullTask Reached!", 2)
        elif self.type is CommandType.MultiTask:
            # Multi Task
            # TODO: Make that behaviour however
            # DEV: Need a loop of runs or something
            pass
        else:
            # Error Case
            self._log.log_verbose("Task not implemented or broken!", 0)
            raise NotImplementedError("Task is Not Implemented or Broken!")
        return None

    def execute_as_singletask_or_multicommand(self):
        """Single Task and Multi Command Task Runner"""
        # Log Starting Run
        self._log.log_verbose("Running {0} Task...".format(self.name), 1)
        # Loop in Commands and their output
        for cmd_in, cmd_out in zip(
                self.commands,
                execute_generator(self.commands)):
            # Log Execute
            self._log.log_verbose('Executing "{0}"...'.format(cmd_in), 2)

            def prefixer(text):
                """Custom prefixer function needed only in that function"""
                return "[{0}]: {1}".format(
                    datetime.now().strftime("%H:%M:%S"),
                    text)
            # Log Output
            self._log.log(
                cmd_out.decode("utf-8"),
                prefixer)
        # Log Finish
        self._log.log_verbose("Finished {0} Task...".format(self.name), 1)

    def execute_as_watcher(self):
        """Watcher Task Runner"""
        # TODO: To be written
        from ..core import NotImplementedFeatureError
        raise NotImplementedFeatureError("Not Implemented Feature Called")

    def execute_as_scheduler(self):
        """Scheduler Task Runner"""
        # TODO: To be written
        from ..core import NotImplementedFeatureError
        raise NotImplementedFeatureError("Not Implemented Feature Called")

    def __add__(self, other):
        """Addition Operator Overload Function"""
        # Asserting that both of objects is Command Class objects
        assert isinstance(other, Command)
        # This is for multitask
        # Changing Type and Mode to MultiTask
        self.type = CommandType.MultiTask
        self.mode = CommandMode.MultiTask_Standard
        self.commands.append(*other.commands)
        self.platform = Platform(self.platform.value | other.platform.value)
        # Arguments
        if self.arguments is None and other.arguments is None:
            self.arguments = None
        elif self.arguments is None and other.arguments is not None:
            self.arguments = other.arguments
        elif self.arguments is not None and other.arguments is None:
            self.arguments = self.arguments
        else:
            self.arguments += other.arguments
        # Special Arguments
        if self.special is None and other.special is None:
            self.special = None
        elif self.special is None and other.special is not None:
            self.special = other.special
        elif self.special is not None and other.special is None:
            self.special = self.special
        else:
            self.special += other.special
        # Returning result
        return self

    def argument_parser(self):
        return self.comment


class CommandList(object):
    """Command List Class
    This class is to make custom list type for Command Class.
    """
    def __init__(self, value: Command) -> None:
        """Constructor of the Class"""
        self.values = value

    def __getitem__(self, key):
        """Dictionary Based Getting Value Overload Function"""
        for i in self.values:
            if i == key or i.name == key:
                return i
        raise KeyError("Key not found!")

    def __setitem__(self, key, value):
        """Dictionary Based Setting Value Overload Function"""
        # To ReInstansiate Values of CommandList
        oldvalues = self.values
        self.values = []
        # New Item Fallback
        item_set = False
        # Loop
        for i in oldvalues:
            if i == key or i.name == key:
                item_set = True
                self.values.append(value)
            else:
                self.values.append(i)
        if item_set is False:
            self.values.append(value)

    def __delitem__(self, key):
        """Dictionary Based Deleting Value Overload Function"""
        oldvalues = self.values
        self.values = []
        for i in oldvalues:
            if i != key and i.name != key:
                self.values.append(i)

    def __contains__(self, item):
        """`in` Operator Overload Function"""
        for i in self.values:
            if i == item or i.name == item:
                return True
        return False

    def items(self):
        return self.values



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""PAiP Web Module - Reading Informations about system

This module creates function to get information about system.

NAME - PAiP Web Module - OS Information
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT
VERSION - v.0.0.0.1
"""
import platform
from collections import namedtuple

def SystemVersion():
    """Printing Debug

    Function to get system information
    Returns:
        namedtuple with information about system
    Return Example:
            machine                 = AMD64
            network_name            = ubuntu
            processor_name          = AMD64 Family 21 Model 48 Stepping 1, AuthenticAMD
            python_compiler         = MSC v.1900 64 bit (AMD64)
            python_implementation   = CPython
            python_version          = 3.6.3
            python_version_tuple    = ('3', '6', '3')
            system_release          = 10
            system_system           = Windows
            system_version          = 10.0.17025
            system_tuple            = ('Windows', '10', '10.0.17025')
            system_uname            = (system='Windows', node='ubuntu', release='10',
                                    version='10.0.17025', machine='AMD64',
                                    processor='AMD64 Family 21 Model 48 Stepping 1, AuthenticAMD')
            platform_info           = ('10', '10.0.17025', 'SP0', 'Multiprocessor Free')
    """
    ver = namedtuple("ver", [
        "machine",
        "network_name",
        "processor_name",
        "python_compiler",
        "python_implementation",
        "python_version",
        "python_version_tuple",
        "system_release",
        "system_system",
        "system_version",
        "system_tuple",
        "system_uname",
        "platform_info"
    ])
    platform_information = ""
    try: # Java Platform
        platform_information = platform.java_ver()
        if platform_information[0] == '':
            raise Exception()
    except Exception:
        try: # Windows Platform
            platform_information = platform.win32_ver()
            if platform_information[0] == '':
                raise Exception()
        except Exception:
            try: # Mac OS Platform
                platform_information = platform.mac_ver()
                if platform_information[0] == '':
                    raise Exception()
            except Exception: # Unknown
                platform_information = ()
    osversion = ver(
        machine=platform.machine(),
        network_name=platform.node(),
        processor_name=platform.processor(),
        python_compiler=platform.python_compiler(),
        python_implementation=platform.python_implementation(),
        python_version=platform.python_version(),
        python_version_tuple=platform.python_version_tuple(),
        system_system=platform.system(),
        system_release=platform.release(),
        system_version=platform.version(),
        system_tuple=platform.system_alias(
            platform.system(),
            platform.release(),
            platform.version()
        ),
        system_uname=platform.uname(),
        platform_info=platform_information
    )
    return osversion



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""PAiP Web Module - Execute Commands

This module creates function to call command line commands.

NAME - PAiP Web Module - Execute Commands
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT
VERSION - v.0.0.0.4
"""
# TODO: MAKE MORE COMPLEX THAT SCRIPT [GENERATOR VERSION]
def execute(command):
    """Function to execute command line commands

    Args:
        command (object): Command

    Returns:
        :obj:`str`: Returning Output of Command
    """
    from subprocess import run, PIPE
    if isinstance(command, list):
        retval = ""
        for cmd in command:
            retval += str(run(cmd, shell=True, stdout=PIPE).stdout)
        return retval
    else:
        return run(command, shell=True, stdout=PIPE).stdout


def execute_generator(command):
    """Function to execute command line commands in generator style

    Args:
        command (object): Command

    Yields:
        :obj:`str`: Returning Output of Command
    """
    from subprocess import run, PIPE
    if isinstance(command, list):
        for cmd in command:
            yield run(cmd, shell=True, stdout=PIPE).stdout
    else:
        yield run(command, shell=True, stdout=PIPE).stdout



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Test

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT

"""
# Documentation
"""
Test 5
================
Tests for v0.4.0dev0 | pwbs.log.logger
================
Test 5.0 - pwbs.log.logger.LoggerAssertionError
Test 5.1 - pwbs.log.logger.BaseLogger
Test 5.2 - pwbs.log.logger.LogLogger
Test 5.3 - pwbs.log.logger.Logger
# Test 5.2,5.3 not covered test_object.log_file_write()
"""
# Test Function


def test_5_0():
    """Testing pwbs.log.logger.LoggerAssertionError"""
    # Import
    from ..log.logger import LoggerAssertionError
    # Tests
    try:
        raise LoggerAssertionError()
        assert False
    except LoggerAssertionError:
        assert True


def test_5_1():
    """Testing pwbs.log.logger.BaseLogger"""
    # Import
    from ..log.logger import BaseLogger, LoggerAssertionError
    # Tests
    test_object = BaseLogger()
    # # Debug
    test_object.log_debug("Test 5.1.1 - You shouldn't see that text immidiately")
    test_object.debug(True)
    test_object.log_debug("Test 5.1.1 - You should see that text immidiately")
    # # Verbose
    test_object.log_verbose("Test 5.1.2 - You shouldn't see that text")
    test_object.verbose(255)
    test_object.log_verbose("Test 5.1.2 - You should see that text")
    # # Log Methods and Variations
    test_object.log("Test 5.1.3 - Log Method")
    test_object.log_wop("Test 5.1.3 - Log Without Prefixer Method")
    test_object.log_assertion(True, "Test 5.1.3 - Assertion")
    try:
        test_object.log_assertion(False, "Test 5.1.3 - Assertion")
        raise NotImplementedError()
    except LoggerAssertionError:
        assert True


def test_5_2():
    """Testing pwbs.log.logger.LogLogger"""
    # Import
    from ..log.logger import LogLogger, LoggerAssertionError
    # Tests
    test_object = LogLogger()
    test_array = test_object.story
    # # Debug
    test_object.log_debug("Test 5.2.1 - You shouldn't see that text immidiately")
    test_object.debug(True)
    test_object.log_debug("Test 5.2.1 - You should see that text immidiately")
    test_array = test_object.story
    # # Verbose
    test_object.log_verbose("Test 5.2.2 - You shouldn't see that text")
    test_array = test_object.story
    test_object.verbose(255)
    test_object.log_verbose("Test 5.2.2 - You should see that text")
    test_array = test_object.story
    # # Log Methods and Variations
    test_object.log("Test 5.2.3 - Log Method")
    test_array = test_object.story
    test_object.log_wop("Test 5.2.3 - Log Without Prefixer Method")
    test_array = test_object.story
    test_object.log_assertion(True, "Test 5.2.3 - Assertion")
    test_array = test_object.story
    try:
        test_object.log_assertion(False, "Test 5.2.3 - Assertion")
        test_array = test_object.story
        raise NotImplementedError()
    except LoggerAssertionError:
        assert True


def test_5_3():
    """Testing pwbs.log.logger.Logger"""
    # Import
    from ..log.logger import Logger, LoggerAssertionError
    # Tests
    test_object = Logger()
    test_array = test_object.story()
    # # # BaseLogger Part
    # # Debug
    test_object.log_debug("Test 5.3.1 - You shouldn't see that text immidiately")
    test_object.debug(True)
    test_object.log_debug("Test 5.3.1 - You should see that text immidiately")
    # # Verbose
    test_object.log_verbose("Test 5.3.2 - You shouldn't see that text")
    test_object.verbose(255)
    test_object.log_verbose("Test 5.3.2 - You should see that text")
    # # Log Methods and Variations
    test_object.log("Test 5.3.3 - Log Method")
    test_object.log_wop("Test 5.3.3 - Log Without Prefixer Method")
    test_object.log_assertion(True, "Test 5.3.3 - Assertion")
    try:
        test_object.log_assertion(False, "Test 5.3.3 - Assertion")
        raise NotImplementedError()
    except LoggerAssertionError:
        assert True
    # # # LogLogger Part
    # # Debug
    test_object.log_debug("Test 5.3.4 - You shouldn't see that text immidiately")
    test_object.debug(True)
    test_object.log_debug("Test 5.3.4 - You should see that text immidiately")
    # # Verbose
    test_object.log_verbose("Test 5.3.5 - You shouldn't see that text")
    test_object.verbose(255)
    test_object.log_verbose("Test 5.3.5 - You should see that text")
    # # Log Methods and Variations
    test_object.log("Test 5.3.6 - Log Method")
    test_object.log_wop("Test 5.3.6 - Log Without Prefixer Method")
    test_object.log_assertion(True, "Test 5.3.6 - Assertion")
    try:
        test_object.log_assertion(False, "Test 5.3.6 - Assertion")
        raise NotImplementedError()
    except LoggerAssertionError:
        assert True



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Core Module
This module contains the most core functionality,
needed by other functions and classes.

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Imports
from datetime import datetime

# Underscore Variables

"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'

# Globals for PWBS


def prefix_text(text=""):
    """Function to prefix text
    Default Prefixer
    Args:
        text (:obj:`str`): Text to prefix
    """
    return "PWBS[{0}]: {1}".format(datetime.now().strftime("%H:%M:%S"), text)


class NotImplementedFeatureError(NotImplementedError):
    """Error for Not Implemented Functionality"""
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""PAiP Web Module - JSON

This module creates function to read and write JSON.

NAME - PAiP Web Module - JSON
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT
VERSION - v.0.0.0.3
"""
### Imports
import os
import json
### Functions

def read_json(filename):
    """Function to read data from json file
    Args:
        filename (object): File Name

    Returns:
        :obj:`str`: Returning Data from JSON file
    """
    dane = {}
    if os.path.isfile(filename):
        with open(filename, "r") as plik:
            dane = json.load(plik)
    return dane

def write_json(filename, data):
    """Function to write data to json file
    Args:
        filename (object): File Name
        data (object): Data

    Returns:
        :obj:`str`: Returning Data from JSON file
    """
    with open(filename, "w") as plik:
        json.dump(data, plik)
    return True

class JSON_IO(object):
    """JSON Input Output Class [Alternative to functions]

    Attributes:
        filename (object): File name
        data (object): data

    """
    def __init__(self, filename):
        """Init Method

        Constractor of class
        Args:
            filename (object): File name
        """
        self.filename = filename
        self.read()
    def read(self):
        """Reading File
        Returns:
            Data from file
        """
        self.data = read_json(self.filename)
        return self.data
    def write(self, newdata):
        """Writing to File
        Args:
            newdata: New Data which you want to be overwrite to file
        Returns:
            Data from file
        """
        write_json(self.filename, newdata)
        return self.read()



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Logger

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Imports
from __future__ import print_function
from ..core import prefix_text

# Class Definition


class LoggerAssertionError(AssertionError):
    """Error to show when Logger.log_assertion failed assertion."""
    pass


class BaseLogger(object):
    """Base Logger Class
    Logging to STDOUT.
    """
    def __init__(self):
        """Constructor of the Class"""
        """Story Log for Delayed Debug"""
        self.debug_delayed_story = []
        # Initiator of Debug Mode
        self.debug()
        # Initiator of Verbose Mode
        self.verbose()

    def debug(self, state=False):
        """Debug Mode Changer
        Args:
            state (bool): State to change to
                Defaults to False [Debug Mode Disabled].
        """
        # Inform when Debug Mode is turned on
        if state:
            print(prefix_text("Debug Mode turned on!"))
            print(prefix_text("[DEBUG]: Delayed Messages"))
            for text in self.debug_delayed_story:
                print(text)

        # Change State Class Variable
        self.debug_state = state

    def verbose(self, state=1):
        """Verbose Mode Changer
        Args:
            state (int): State to change to
                Defaults to 1.
                Levels:
                    0: No Verbose
                    1: [Default] Small Verbose
                    2: Medium Verbose
                    3: Full Verbose
                    255: Extreme Verbose Mode
                        Debug Verbose Mode
        """
        # Change State Class Variable
        self.verbose_state = state
        # Log that verbose level changed
        self.log_verbose("Verbose Mode is now set to: {0}".format(state), 2)

    def log(self, text, prefix=prefix_text):
        """Log Function
        Args:
            text: Text to log
            prefix (function): Prefixer
                Defaults to Default PWBS Prefixer.
        """
        return print(prefix(text))

    def log_wop(self, text):
        """Log Function Without Prefixer
        Args:
            text: Text to log
        """
        return print(text)

    def log_verbose(self, text, verbose=1):
        """Log Verbose Function
        Args:
            text: Text to log
            verbose (int): Level to see that
                Defaults to 1.
        """
        # Checking is Verbose Level of message is good to log it
        if self.verbose_state >= verbose:
            return self.log(text)
        # Logging Debug info about message cannot be logged.
        return self.log_debug(
            "Verbose Logger doesn't succeed to log by set verbose level."
        )

    def log_debug(self, text):
        """Log Debug Function
        Args:
            text: Text to log
        """
        # Checking is Debug Mode Activated and logging
        if self.debug_state:
            return self.log("[DEBUG]: {0}".format(text))
        else:
            self.debug_delayed_story.append(
                prefix_text("[DEBUG]: {0}".format(text)))
        return None

    def log_assertion(self, assertion, name=None):
        """Log Assertion Function
        Args:
            assertion (bool): Assertion to test
            name (optional): Name of the assertion
        """
        # Try Assertion
        try:
            # Check Assertion
            assert assertion
            # If succeed then log good info
            if name is None:
                self.log_debug("Assertion succeed!")
            else:
                self.log_debug('Assertion "{0}" succeed!'.format(name))
        except AssertionError as e:
            # Assertion failed then log bad info
            if name is None:
                self.log_debug("Assertion failed!")
            else:
                self.log_debug('Assertion "{0}" failed!'.format(name))
            # ReRaise AsserionError
            raise LoggerAssertionError('Assertion failed!') from e


class LogLogger(BaseLogger):
    """Base Logger Class
    Logging to variable and can to file.
    """
    def __init__(self):
        """Constructor of the Class"""
        """Story Log Variable"""
        self.story = []
        """Log File Variable"""
        self.logfile = "pwbs.log"
        """Active Logging"""
        self.activelogging = False
        """Running Base Logger Constructor"""
        super().__init__()

    def debug(self, state=False):
        """Debug Mode Changer
        Args:
            state (bool): State to change to
                Defaults to False [Debug Mode Disabled].
        """
        # Inform when Debug Mode is turned on
        if state:
            self.story.append(prefix_text("Debug Mode turned on!"))
            self.story.append(prefix_text("[DEBUG]: Delayed Messages"))
            for item in self.debug_delayed_story:
                self.story.append(item)
        # Change State Class Variable
        self.debug_state = state
        # Active Logging
        if self.activelogging:
            self.log_file_write()

    def verbose(self, state=1):
        """Verbose Mode Changer
        Args:
            state (int): State to change to
                Defaults to 1.
                Levels:
                    0: No Verbose
                    1: [Default] Small Verbose
                    2: Medium Verbose
                    3: Full Verbose
                    255: Extreme Verbose Mode
                        Debug Verbose Mode
        """
        # Change State Class Variable
        self.verbose_state = state
        # Log that verbose level changed
        self.log_verbose("Verbose Mode is now set to: {0}".format(state), 2)
        # Active Logging
        if self.activelogging:
            self.log_file_write()

    def log(self, text, prefix=prefix_text, story=None):
        """Log Function
        Args:
            text: Text to log
            prefix (function): Prefixer
                Defaults to Default PWBS Prefixer.
            story: Story to write to
                Defaults to Default Story
        """
        if story is None:
            story = self.story
        ret = story.append(prefix(text))
        # Active Logging
        if self.activelogging:
            self.log_file_write()
        return ret

    def log_wop(self, text):
        """Log Function Without Prefixer
        Args:
            text: Text to log
        """
        ret = self.story.append(text)
        # Active Logging
        if self.activelogging:
            self.log_file_write()
        return ret

    def log_verbose(self, text, verbose=1):
        """Log Verbose Function
        Args:
            text: Text to log
            verbose (int): Level to see that
                Defaults to 1.
        """
        # Checking is Verbose Level of message is good to log it
        if self.verbose_state >= verbose:
            ret = self.log(text)
        # Logging Debug info about message cannot be logged.
        ret = self.log_debug(
            "Verbose Logger doesn't succeed to log by set verbose level."
        )
        # Active Logging
        if self.activelogging:
            self.log_file_write()
        return ret

    def log_debug(self, text):
        """Log Debug Function
        Args:
            text: Text to log
        """
        # Checking is Debug Mode Activated and logging
        if self.debug_state:
            ret = self.log("[DEBUG]: {0}".format(text))
        else:
            self.log(
                "[DEBUG]: {0}".format(text),
                story=self.debug_delayed_story)
        ret = None
        # Active Logging
        if self.activelogging:
            self.log_file_write()
        return ret

    def log_assertion(self, assertion, name=None):
        """Log Assertion Function
        Args:
            assertion (bool): Assertion to test
            name (optional): Name of the assertion
        """
        # Try Assertion
        try:
            # Check Assertion
            assert assertion
            # If succeed then log good info
            if name is None:
                self.log_debug("Assertion succeed!")
            else:
                self.log_debug('Assertion "{0}" succeed!'.format(name))
            # Active Logging
            if self.activelogging:
                self.log_file_write()
        except AssertionError as e:
            # Assertion failed then log bad info
            if name is None:
                self.log_debug("Assertion failed!")
            else:
                self.log_debug('Assertion "{0}" failed!'.format(name))
            # Active Logging
            if self.activelogging:
                self.log_file_write()
            # ReRaise AsserionError
            raise LoggerAssertionError('Assertion failed!') from e

    def log_file_write(self, file=None):
        """Log File Writer
        Args:
            file (:obj:`str`): Filename to write log
        """
        #print(self.story)
        if file is None:
            file = self.logfile
        with open(file, mode="w") as f:
            f.writelines([x+"\n" for x in self.story])


class Logger(BaseLogger):
    """Logger Class
    Logging to variable and to STDOUT.
    """
    def __init__(self):
        """Constructor of the Class"""
        """Variable Logger"""
        self.log_logger = LogLogger()
        """Locker"""
        self.locker_log_logger = False
        """Running Base Logger Constructor"""
        super().__init__()

    def debug(self, state=False):
        """Debug Mode Changer
        Args:
            state (bool): State to change to
                Defaults to False [Debug Mode Disabled].
        """
        self.log_logger.debug(state)
        return super().debug(state)

    def verbose(self, state=1):
        """Verbose Mode Changer
        Args:
            state (int): State to change to
                Defaults to 1.
                Levels:
                    0: No Verbose
                    1: [Default] Small Verbose
                    2: Medium Verbose
                    3: Full Verbose
                    255: Extreme Verbose Mode
                        Debug Verbose Mode
        """
        self.log_logger.verbose(state)
        self.locker_log_logger = True
        ret = super().verbose(state)
        self.locker_log_logger = False
        return ret

    def log(self, text, prefix=prefix_text):
        """Log Function
        Args:
            text: Text to log
            prefix (function): Prefixer
                Defaults to Default PWBS Prefixer.
        """
        if not self.locker_log_logger:
            self.log_logger.log(text, prefix)
        return super().log(text, prefix)

    def log_wop(self, text):
        """Log Function Without Prefixer
        Args:
            text: Text to log
        """
        if not self.locker_log_logger:
            self.log_logger.log_wop(text)
        return super().log_wop(text)

    def log_verbose(self, text, verbose=1):
        """Log Verbose Function
        Args:
            text: Text to log
            verbose (int): Level to see that
                Defaults to 1.
        """
        if not self.locker_log_logger:
            self.log_logger.log_verbose(text, verbose)
        self.locker_log_logger = True
        ret = super().log_verbose(text, verbose)
        self.locker_log_logger = False
        return ret

    def log_debug(self, text):
        """Log Debug Function
        Args:
            text: Text to log
        """
        if not self.locker_log_logger:
            self.log_logger.log_debug(text)
        self.locker_log_logger = True
        ret = super().log_debug(text)
        self.locker_log_logger = False
        return ret

    def log_assertion(self, assertion, name=None):
        """Log Assertion Function
        Args:
            assertion (bool): Assertion to test
            name (optional): Name of the assertion
        """
        self.log_logger.log_assertion(assertion, name)
        self.locker_log_logger = True
        ret = super().log_assertion(assertion, name)
        self.locker_log_logger = False
        return ret

    def log_file_write(self, file=None):
        """Log File Writer
        Args:
            file (:obj:`str`): Filename to write log
        """
        return self.log_logger.log_file_write(file)

    def story(self):
        return self.log_logger.story



# -*- coding: utf-8 -*-
"""PAiP Web Build System - API Module

This module is public api of PWBS to make it easier to integrate to other programs and for plugins of PWBS.

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Underscore Variables

"""Author of the module"""
__author__ = 'Patryk Adamczyk'
"""Module License"""
__license__ = 'MIT'
"""Documentation format"""
__docformat__ = 'restructuredtext en'



# -*- coding: utf-8 -*-
"""PAiP Web Build System - PWBS Config Manager

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paipweb.com>
LICENSE - MIT

"""
# Imports
from __future__ import print_function
from ..log.logger import Logger
from .config_manager import ConfigManager, PWBSInvalidConfigFile
from ..command.command import Command, CommandList
from ..command.command import CommandType, Platform, CommandMode

# Class Definition


class PWBS_ConfigManager(object):
    """PWBS Config Class"""
    def __init__(self):
        """Constructor of the Class"""
        # Variables
        """Logger"""
        self.log = Logger()
        """Verbose Level"""
        self.verbose = 1
        """Debug Mode"""
        self.debug = False
        """Config File Manager"""
        self.configmanager = ConfigManager()
        """Commands"""
        self.commands = CommandList([])
        # Constructor Methods
        self.config_file()

    def config_file(self):
        """Config File Function
        Returns:
            JSON Object from Config File
        """
        self.log.log_debug("Loading Configuration File Data...")
        data = self.configmanager.load()
        self.log.log_debug("Configuration File Data Loaded.")
        return data

    def commands_to_commandlist(self):
        commands = []
        """Testing for invalid Configuration File"""
        try:
            assert self.config_file()["commands"] is not None
        except AssertionError as e:
            raise PWBSInvalidConfigFile("Invalid Configuration File") from e
        except KeyError as e:
            raise PWBSInvalidConfigFile("Invalid Configuration File") from e
        """Loop of creating Commands"""
        for name, body in self.config_file()["commands"].items():
            if isinstance(body, str):
                # Single Task
                commands.append(Command(
                    name,
                    CommandType.SingleTask,
                    [body],
                    mode=CommandMode.SingleTask_Standard))
            elif isinstance(body, list):
                # TODO: Multi Task
                # from ..core import NotImplementedFeatureError
                # raise NotImplementedFeatureError("Not Implemented Feature Called")
                pass
            else:
                # Everything Else
                try:
                    cmdtype = PWBS_ConfigManager.ctcl__cmdtype(body)
                except KeyError:
                    cmdtype = CommandType.ErrorTask
                try:
                    comment = PWBS_ConfigManager.ctcl__comment(body)
                except KeyError:
                    comment = ""
                try:
                    cmdmode = PWBS_ConfigManager.ctcl__cmdmode(body)
                except KeyError:
                    cmdmode = CommandMode.ErrorTask_ErrorMode
                try:
                    cmds = PWBS_ConfigManager.ctcl__commands(body)
                except KeyError:
                    cmds = None
                try:
                    arguments = PWBS_ConfigManager.ctcl__arguments(body)
                except KeyError:
                    arguments = None
                try:
                    platform = PWBS_ConfigManager.ctcl__platform(body)
                except KeyError:
                    platform = Platform.Any
                commands.append(Command(
                    name,
                    cmdtype,
                    cmds,
                    comment,
                    cmdmode,
                    arguments,
                    None,
                    platform))
        self.commands = CommandList(commands)

    def ctcl__cmdtype(commandbody):
        ct = commandbody["mode"]
        if ct == "st":
            return CommandType.SingleTask
        elif ct == "mt":
            return CommandType.MultiTask
        elif ct == "mc":
            return CommandType.MultiCommandTask
        elif ct == "wc0" or ct == "wc":
            return CommandType.WatcherTask
        elif ct == "sc0" or ct == "sc":
            return CommandType.SchedulerTask
        return CommandType.ErrorTask

    def ctcl__comment(commandbody):
        return commandbody["comment"]

    def ctcl__cmdmode(commandbody):
        ct = commandbody["mode"]
        if ct == "st":
            return CommandMode.SingleTask_Standard
        elif ct == "mt":
            return CommandMode.MultiTask_Standard
        elif ct == "mc":
            return CommandMode.MultiCommandTask_Standard
        elif ct == "wc0":
            return CommandMode.WatcherTask_StartAndRun
        elif ct == "wc":
            return CommandMode.WatcherTask_StartAndWait
        elif ct == "sc0":
            return CommandMode.SchedulerTask_StartAndRun
        elif ct == "sc":
            return CommandMode.SchedulerTask_StartAndWait
        return CommandMode.ErrorTask_ErrorMode

    def ctcl__arguments(commandbody):
        return commandbody["context"]

    def ctcl__commands(commandbody):
        return commandbody["commands"]

    def ctcl__platform(commandbody):
        ct = commandbody["platform"]
        if isinstance(ct, int):
            return Platform(ct)
        elif isinstance(ct, str):
            if ct.lower() == "windows" or ct.lower() == "win":
                return Platform.Windows
            elif ct.lower() == "linux" or ct.lower() == "lin":
                return Platform.Linux
            elif ct.lower() == "macos":
                return Platform.MacOS
            elif ct.lower() == "macosx":
                return Platform.MacOS
            elif ct.lower() == "mac":
                return Platform.MacOS
            else:
                return Platform.Other
        else:
            raise ValueError("Not Supported Type for that operation")



# -*- coding: utf-8 -*-
"""PAiP Web Build System - Test

NAME - PAiP Web Build System
AUTHOR - Patryk Adamczyk <patrykadamczyk@paip.com.pl>
LICENSE - MIT

"""
# Documentation
"""
Test 3
================
Tests for v0.4.0dev0 | pwbs.core
================
Test 3.0 - pwbs.core.NotImplementedFeatureError
Test 3.1 - pwbs.core.prefix_text | TO_TEST: BAD TEST - Temporary Solution
"""
# Test Function


def test_3_0():
    """Testing pwbs.core.NotImplementedFeatureError"""
    # Import
    from ..core import NotImplementedFeatureError
    # Test
    try:
        raise NotImplementedFeatureError()
        assert False
    except NotImplementedFeatureError:
        assert True

def test_3_1():
    """Testing pwbs.core.prefix_text"""
    # Import
    from ..core import prefix_text
    from datetime import datetime
    # Values
    assert prefix_text(10) == "PWBS[{0}]: {1}".format(datetime.now().strftime("%H:%M:%S"), 10)
